
Card = Core.class(Bitmap)

-- create new card
function Card:init(texture, options)
  self.suite  = "spades"
  self.value  = 1
  self.faceUp = true
  self.discard = false
  
	if options then
		for key, value in pairs(options) do
			self[key]= value
		end
	end
  
  self:setAnchorPoint(0.5, 0.5)
  
  local rX = self:getWidth() * 0.5 -- half width and half height
  local rY = self:getHeight() * 0.5

  self.original = { -- original vertices
    {x = - rX, y = - rY},
    {x = - rX, y =  rY},
    {x = rX, y =  rY,},
    {x = rX, y =  - rY},
  }

  self.transformed = { -- transformed vertices
    {x = 0, y = 0},
    {x = 0, y = 0},
    {x = 0, y = 0},
    {x = 0, y = 0},
  }
end

function Card:getName()
  local names = {"ace", "two", "three", "four", "five", "six", "seven", "eight", "nine", "ten", "jack", "queen", "king", "joker", }
  return(names[self.value] .. " of " .. self.suite)
end

-- set card screen position
function Card:setCardPosition(x, y)
  local ox, oy = self:getPosition()
  local verts = self.transformed
  for i = 1, #verts do
    local v = verts[i]
    v.x = (v.x - ox) + x -- recalculate x position
    
    v.y = (v.y - oy) + y
  end
  self:setPosition(x, y) -- set image position
end

-- set card rotation
function Card:setCardRotation(angle)
  local function add(x1, y1, x2, y2) -- add two vectors
    return x1 + x2, y1 + y2
  end

  local function rotate(x, y, angle) -- rotate a point about 0, 0
    local sin, cos = math.sin, math.cos
    local c, s = cos(^<angle), sin(^<angle)
    return c * x - s * y, s * x + c * y
  end

  local px, py = self:getPosition()

  local o = self.original
  local t = self.transformed
  for i = 1, #o do
    local ov, tv = o[i], t[i] -- next vertices to rotate
    tv.x, tv.y = add(px, py, rotate(ov.x, ov.y, angle)) -- rotate and position vertice
  end

  self:setRotation(angle) -- set image rotation
end

-- return true if the coordinate resides inside the polygon
-- This function from Hardon Collider (https://github.com/vrld/HC), Copyright (c) 2011 Matthias Richter
function Card:containsPoint(x, y)
  
  -- test if an edge cuts the ray
  local function cut_ray(p,q)
    return ((p.y > y and q.y < y) or (p.y < y and q.y > y)) -- possible cut
    and (x - p.x < (y - p.y) * (q.x - p.x) / (q.y - p.y)) -- x < cut.x
  end

  -- test if the ray crosses boundary from interior to exterior.
  -- this is needed due to edge cases, when the ray passes through
  -- polygon corners
  local function cross_boundary(p,q)
    return (p.y == y and p.x > x and q.y < y)
    or (q.y == y and q.x > x and p.y < y)
  end

  local v = self.transformed
  local in_polygon = false
  local p, q = v[#v], v[#v]
  for i = 1, #v do
    p, q = q, v[i]
    if cut_ray(p, q) or cross_boundary(p, q) then
      in_polygon = not in_polygon
    end
  end
  return in_polygon
end
